@@ -107,7 +107,7 @@ def get_type_of_parent(identifier):
             current_type = TYPES[name]['type']
             for index in range(1, len(identifier)):
                 # TODO find type...
-                print '** INCOMPLETE FEATURE !ID=', identifier[index]
+                print '** INCOMPLETE FEATURE ** '
             kind = current_type['Kind']
     return kind, name
 
@@ -121,6 +121,35 @@ def traceability(symbol):
     return trace
 
 
+def write_statement(param, newline):
+    ''' Generate the code for the special "write" operator '''
+    code = []
+    string = ''
+    local = []
+    basic_type = find_basic_type(param.exprType) or {}
+    type_kind = basic_type.get('Kind')
+    if type_kind == 'StringType':
+        # Raw string
+        string = '"' + param.inputString[1:-1] + '"'
+    elif type_kind in ('IntegerType', 'RealType', 'BooleanType'):
+        code, string, local = decipher_expression(param)
+        if type_kind == 'IntegerType':
+            cast = "Interfaces.Integer_64"
+        elif type_kind == 'RealType':
+            cast = 'Long_Float'
+        elif type_kind == 'BooleanType':
+            cast = 'Boolean'
+        string = "{cast}'Image({s})".format(cast=cast, s=string)
+    else:
+        print('[ERROR] Unsupported parameter in write call' +
+                param.var.inputString)
+        return code, string, local
+    code.append('Put{line}({string});'.format(
+        line='_Line' if newline else '',
+        string=string))
+    return code, string, local
+
+
 def output_statement(output):
     ''' Generate the code of a set of output or procedure call statement '''
     code = []
@@ -134,12 +163,18 @@ def output_statement(output):
 
         if signal_name.lower() in ('write', 'writeln'):
             # special built-in SDL procedure for printing strings
-            # TODO = decipherWrite function that prints any ASN.1 value
-            for p in o['params']:
-                code.append('{put}("{string}");'.format(
-                    put='Put' if signal_name.lower() == 'write'
-                    else 'Put_Line',
-                    string=p.inputString[1:-1]))
+            # supports printing of native types (int, real, bool)
+            # but not yet complex ASN.1 structures (sequence/seqof/choice)
+            for param in o['params'][:-1]:
+                stmts, _, local = write_statement(param, newline=False)
+                code.extend(stmts)
+                local_decl.extend(local)
+            for param in o['params'][-1:]:
+                # Last parameter - add newline if necessary
+                stmts, _, local = write_statement(param, newline=True if
+                        signal_name.lower() == 'writeln' else False)
+                code.extend(stmts)
+                local_decl.extend(local)
             continue
         if output.kind == 'output':
             out_sig = OUT_SIGNALS.get(signal_name.lower())
@@ -614,10 +649,13 @@ def transition(tr):
         code.extend(stmt)
         local_decl.extend(local_var)
     if tr.terminator:
+        code.extend(traceability(tr.terminator))
         if tr.terminator.label:
             code.append('<<{label}>>'.format(
                 label=tr.terminator.label.inputString))
-        if tr.terminator.kind == 'next_state':
+        if(tr.terminator.kind == 'next_state' and
+                tr.terminator.inputString.strip() != '-'):
+            # discard the "-" state (remain in the same state)
             code.append('state := {nextState};'.format(
                 nextState=tr.terminator.inputString))
             code.append('return;')
@@ -693,8 +731,8 @@ def generate(process):
         for stateName, inputSymbols in process.mapping.viewitems():
             if stateName != 'START':
                 for i in inputSymbols:
-                    if inputSignal.lower() in [inp.lower() for
-                                               inp in i.inputlist]:
+                    if inputSignal.lower() in (inp.lower() for
+                                               inp in i.inputlist):
                         mapping[inputSignal][stateName] = i
 
     # Generate the TASTE template
