@@ -85,18 +85,18 @@ class Completer(QGraphicsProxyWidget):
         widget = QListWidget()
         super(Completer, self).__init__(parent.parentItem())
         self.setWidget(widget)
-        self.c = QCompleter(parent.parentItem().completionList())
+        self.c = QCompleter(parent.parentItem().completion_list)
         self.c.setCaseSensitivity(Qt.CaseInsensitive)
         self.parent = parent
 
-    def set_completion_prefix(self, completionPrefix):
+    def set_completion_prefix(self, completion_prefix):
         '''
         Set the current completion prefix (user-entered text)
         and set the corresponding list of words in the popup widget
         '''
         if not hasattr(self, 'c'):
             return 0
-        self.c.setCompletionPrefix(completionPrefix)
+        self.c.setCompletionPrefix(completion_prefix)
         self.widget().clear()
         for i in xrange(self.c.completionCount()):
             self.c.setCurrentRow(i)
@@ -180,10 +180,11 @@ class EditableText(QGraphicsTextItem):
                 Qt.TextEditorInteraction
                 | Qt.LinksAccessibleByMouse
                 | Qt.LinksAccessibleByKeyboard)
-        self.completerHasFocus = False
+        self.completer_has_focus = False
         self.editing = False
         self.try_resize()
         self.highlighter = Highlighter(self.document())
+        self.completion_prefix = ''
         self.set_textbox_position()
         self.set_text_alignment()
 
@@ -239,13 +240,13 @@ class EditableText(QGraphicsTextItem):
             invoked when selection is made
         '''
         tc = self.textCursor()
-        extra = len(item.text()) - len(self.completionPrefix)
+        extra = len(item.text()) - len(self.completion_prefix)
         if extra > 0:
             tc.movePosition(QTextCursor.Left)
             tc.movePosition(QTextCursor.EndOfWord)
             tc.insertText(item.text()[-extra:])
             self.setTextCursor(tc)
-        self.completerHasFocus = False
+        self.completer_has_focus = False
         self.completer.hide()
 
     def keyPressEvent(self, event):
@@ -256,16 +257,17 @@ class EditableText(QGraphicsTextItem):
         self.try_resize()
         tc = self.textCursor()
         tc.select(QTextCursor.WordUnderCursor)
-        self.completionPrefix = tc.selectedText()
-        if(hasattr(self.parentItem(), 'completionList') and not
-                self.completer):
+        self.completion_prefix = tc.selectedText()
+        #if(hasattr(self.parentItem(), 'completion_list') and not
+        #        self.completer):
+        if not self.completer:
             self.completer = Completer(self)
             self.completer.widget().itemActivated.connect(
                     self.completion_selected)
         if self.completer:
-            completionCount = self.completer.set_completion_prefix(
-                    self.completionPrefix)
-            if completionCount > 0 and (len(self.completionPrefix) > 1 or
+            completion_count = self.completer.set_completion_prefix(
+                    self.completion_prefix)
+            if completion_count > 0 and (len(self.completion_prefix) > 1 or
                     event.key() == Qt.Key_F8):
                 # Computing the position of the completer
                 # No direct Qt function for that.. doing it the hard way
@@ -287,9 +289,9 @@ class EditableText(QGraphicsTextItem):
                 self.completer.setFocusProxy(None)
                 self.completer.hide()
                 self.setFocus()
-            self.completerHasFocus = False
+            self.completer_has_focus = False
             if self.completer.isVisible() and event.key() == Qt.Key_Down:
-                self.completerHasFocus = True
+                self.completer_has_focus = True
                 self.completer.setFocusProxy(None)
                 self.completer.widget().setFocus()
 
@@ -301,22 +303,22 @@ class EditableText(QGraphicsTextItem):
         '''
         if not self.editing:
             return super(EditableText, self).focusOutEvent(event)
-        if self.completer and not self.completerHasFocus:
+        if self.completer and not self.completer_has_focus:
             self.completer.hide()
             self.completer.resize(0, 0)
         if not self.completer or not self.completer.isVisible():
             if self.oldSize != self.parentItem().boundingRect():
                 undoCmd = ResizeSymbol(self.parentItem(), self.oldSize,
                         self.parentItem().boundingRect())
-                self.scene().undoStack.push(undoCmd)
+                self.scene().undo_stack.push(undoCmd)
                 self.parentItem().cam(self.parentItem().pos(),
                         self.parentItem().pos())
             if self.oldText != str(self):
                 undoCmd = ReplaceText(self, self.oldText, str(self))
-                self.scene().undoStack.push(undoCmd)
-                self.scene().undoStack.endMacro()
+                self.scene().undo_stack.push(undoCmd)
+                self.scene().undo_stack.endMacro()
             else:
-                self.scene().undoStack.endMacro()
+                self.scene().undo_stack.endMacro()
             self.editing = False
             tc = self.textCursor()
             if tc.hasSelection():
@@ -330,25 +332,28 @@ class EditableText(QGraphicsTextItem):
 
     def focusInEvent(self, event):
         ''' When user starts editing text, save previous state for Undo '''
+        super(EditableText, self).focusInEvent(event)
+        # Clear selection otherwise the "Delete" key may delete other items
+        self.scene().clearSelection()
         # Set width to auto-expand, and disables alignment, while editing:
         self.setTextWidth(-1)
         if self.completer:
             self.completer.resize(0, 0)
         if not self.editing:
-            self.scene().undoStack.beginMacro('Edit text')
+            self.scene().undo_stack.beginMacro('Edit text')
             self.oldText = str(self)
             self.oldSize = self.parentItem().boundingRect()
             self.editing = True
-        super(EditableText, self).focusInEvent(event)
+        # super(EditableText, self).focusInEvent(event)
 
-    def mousePressEvent(self, event):
-        '''
-            When the user clicks on the text symbol (to edit),
-            the list of selected objects on the diagram must be cleared
-            to prevent the the "Delete" key from removing the selected objects
-        '''
-        self.scene().clearSelection()
-        super(EditableText, self).mousePressEvent(event)
+#    def mousePressEvent(self, event):
+#        '''
+#            When the user clicks on the text symbol (to edit),
+#            the list of selected objects on the diagram must be cleared
+#            to prevent the the "Delete" key from removing the selected objects
+#        '''
+#        self.scene().clearSelection()
+#        super(EditableText, self).mousePressEvent(event)
 
     def __repr__(self):
         '''
@@ -476,14 +481,17 @@ class Connection(QGraphicsPathItem):
 #            self.childRect = self.child.sceneBoundingRect()
 #        return super(Connection, self).boundingRect()
 
-#    def paint(self, painter, _, __):
-#        ''' Draw the connection '''
-#        self.update_lines()
-#        super(Connection, self).paint(painter, _, __)
+    def paint(self, painter, _, __):
+        ''' Draw the connection '''
+        self.update_lines()
+        super(Connection, self).paint(painter, _, __)
 
 
 class Symbol(QGraphicsPolygonItem):
     ''' Top-level class used to handle all SDL symbols '''
+    # Symbols of a given type share a text-autocompletion list:
+    completion_list = []
+    # Symbols of a given type can be graphically followed to:
     allowed_followers = []
 
     def __init__(self, parent=None):
@@ -509,7 +517,7 @@ class Symbol(QGraphicsPolygonItem):
         # branch entry point allows working with aligned symbols
         self.branchEntryPoint = None
         # Terminal symbol can be used to identify last items of a branch
-        self.terminalSymbol = False
+        self._terminal_symbol = False
         # default textbox alignment: centered in the symbol
         # can differ on specific symbols (e.g. TextArea or label)
         self.textbox_alignment = Qt.AlignCenter
@@ -517,24 +525,35 @@ class Symbol(QGraphicsPolygonItem):
         self.text_alignment = Qt.AlignLeft
         self.setFlags(
                 QGraphicsItem.ItemIsMovable
-                | QGraphicsItem.ItemIsSelectable
-                | QGraphicsItem.ItemSendsGeometryChanges)
+                | QGraphicsItem.ItemIsSelectable)
+#                | QGraphicsItem.ItemSendsGeometryChanges)
 #                | QGraphicsItem.ItemSendsScenePositionChanges)
 
-    def itemChange(self, change, value):
-        '''
-            Function called by Qt when anything happens to the symbol
-            Used to detect when connection lines need to be re-computed
-        '''
-        if change in (QGraphicsItem.GraphicsItemChange.ItemPositionChange,
-                      QGraphicsItem.GraphicsItemChange.ItemPositionHasChanged,
-                      QGraphicsItem.GraphicsItemChange.ItemChildAddedChange):
-            for cnx in self.connections():
-                cnx.update_lines()
-            if self.hasParent:
-                for cnx in self.parentItem().connections():
-                    cnx.update_lines()
-        return super(Symbol, self).itemChange(change, value)
+# Performance of itemChange is not good enough - commented out
+#    def itemChange(self, change, value):
+#        '''
+#            Function called by Qt when anything happens to the symbol
+#            Used to detect when connection lines need to be re-computed
+#        '''
+#        if change in (QGraphicsItem.GraphicsItemChange.ItemPositionChange,
+#                      QGraphicsItem.GraphicsItemChange.ItemPositionHasChanged,
+#                      QGraphicsItem.GraphicsItemChange.ItemChildAddedChange):
+#            for cnx in self.connections():
+#                cnx.update_lines()
+#            if self.hasParent:
+#                for cnx in self.parentItem().connections():
+#                    cnx.update_lines()
+#        return super(Symbol, self).itemChange(change, value)
+
+    @property
+    def terminalSymbol(self):
+        ''' Way to determine if a symbol is terminal (useful for branches) '''
+        return self._terminal_symbol
+
+    @terminalSymbol.setter
+    def terminalSymbol(self, value):
+        ''' Attribute is set per symbol '''
+        self._terminal_symbol = value
 
     def __str__(self):
         ''' Print the text inside the symbol '''
@@ -644,7 +663,7 @@ class Symbol(QGraphicsPolygonItem):
 
     def connections(self):
         ''' Return all child connections of this symbol '''
-        return [c for c in self.childItems() if isinstance(c, Connection)]
+        return (c for c in self.childItems() if isinstance(c, Connection))
 
     def loadHyperlinkDialog(self):
         ''' Load dialog from ui file for defining hyperlink '''
@@ -714,7 +733,7 @@ class Symbol(QGraphicsPolygonItem):
 
     def childSymbols(self):
         ''' Return the list of child symbols, excluding text/connections '''
-        return [item for item in self.childItems() if isinstance(item, Symbol)]
+        return (item for item in self.childItems() if isinstance(item, Symbol))
 
     def resizeItem(self, rect):
         ''' resize item, e.g. when editing text - move children accordingly '''
@@ -724,8 +743,8 @@ class Symbol(QGraphicsPolygonItem):
         polygon = self.computePolygon(rect.width(), rect.height())
         self.setPolygon(polygon)
         # Align children properly when resizing
-        for child in [c for c in self.childItems()
-                if not isinstance(c, Connection)]:
+        for child in (c for c in self.childItems()
+                if not isinstance(c, Connection)):
             # includes position of the text inside the box
             child.moveBy(-deltaX, 0)
         for child in self.childSymbols():
@@ -741,6 +760,7 @@ class Symbol(QGraphicsPolygonItem):
             Handle resizing and moving of items when grabbing
             the lower right corner
         '''
+        # super(Symbol, self).mousePressEvent(event)
         # Save current position to be able to revert move
         self.coord = self.pos()
         eventPos = event.pos()
@@ -772,18 +792,19 @@ class Symbol(QGraphicsPolygonItem):
     def mouseReleaseEvent(self, event):
         ''' Default action when mouse is released: reset mode '''
         if self.mode == 'Resize':
-            self.scene().undoStack.beginMacro('Resize symbol')
+            pass
+            self.scene().undo_stack.beginMacro('Resize symbol')
             undoCmd = ResizeSymbol(self, self.oldRect, self.boundingRect())
-            self.scene().undoStack.push(undoCmd)
+            self.scene().undo_stack.push(undoCmd)
             self.cam(self.coord, self.pos())
             self.updateConnectionPoints()
-            self.scene().undoStack.endMacro()
+            self.scene().undo_stack.endMacro()
         elif self.mode == 'Move':
-            self.scene().undoStack.beginMacro('Move symbol')
+            self.scene().undo_stack.beginMacro('Move symbol')
             undoCmd = MoveSymbol(self, self.coord, self.pos())
-            self.scene().undoStack.push(undoCmd)
+            self.scene().undo_stack.push(undoCmd)
             self.cam(self.coord, self.pos())
-            self.scene().undoStack.endMacro()
+            self.scene().undo_stack.endMacro()
         self.mode = ''
 
     def updateConnectionPoints(self):
@@ -828,7 +849,7 @@ class Symbol(QGraphicsPolygonItem):
             rect.adjust(delta.x(), delta.y(), delta.x(), delta.y())
             self.setPos(newPos)
             undoCmd = MoveSymbol(self, oldPos, newPos)
-            self.scene().undoStack.push(undoCmd)
+            self.scene().undo_stack.push(undoCmd)
 
         # Get all items in the rectangle when placed at the new position
         items = self.scene().items(rect)
@@ -952,11 +973,11 @@ class Comment(Symbol):
         self.prepareGeometryChange()
         polygon = self.computePolygon(rect.width(), rect.height())
         self.setPolygon(polygon)
-        for cnx in self.connections():
-            cnx.update_lines()
-        if self.hasParent:
-            for cnx in self.parent.connections():
-                cnx.update_lines()
+        #for cnx in self.connections():
+        #    cnx.update_lines()
+        #if self.hasParent:
+        #    for cnx in self.parent.connections():
+        #        cnx.update_lines()
 
     def computePolygon(self, width, height):
         polygon = QPolygon(QRect(0, 0, width, height))
@@ -983,16 +1004,15 @@ class Comment(Symbol):
 
     def mouseMoveEvent(self, event):
         ''' Handle item move '''
+        super(Comment, self).mouseMoveEvent(event)
         if self.mode == 'Move':
-            eventPos = event.pos()
-            newY = self.pos().y() + (eventPos.y() - event.lastPos().y())
-            newX = self.pos().x() + (eventPos.x() - event.lastPos().x())
-            if newX < self.parentItem().boundingRect().width() + 5:
+            new_y = self.pos().y() + (event.pos().y() - event.lastPos().y())
+            new_x = self.pos().x() + (event.pos().x() - event.lastPos().x())
+            if new_x < self.parentItem().boundingRect().width() + 5:
                 self.onTheRight = False
             else:
                 self.onTheRight = True
-            self.setPos(newX, newY)
-        super(Comment, self).mouseMoveEvent(event)
+            self.setPos(new_x, new_y)
 
     def mouseReleaseEvent(self, event):
         '''
@@ -1062,7 +1082,7 @@ class HorizontalSymbol(Symbol):
                 sOldPos = s.pos()
                 s.setX(sX)
                 undoCmd = MoveSymbol(s, sOldPos, s.pos())
-                self.scene().undoStack.push(undoCmd)
+                self.scene().undo_stack.push(undoCmd)
             mostLeft = min([s.x() for s in siblings]) if siblings else 0
             if siblings:
                 posX = mostLeft + groupW + 20
@@ -1120,9 +1140,9 @@ class HorizontalSymbol(Symbol):
                 lastCnx[0].setParentItem(self.lastBranchItem)
                 lastCnx = None
             # Get all siblings (surrounding inputs/decision answers)
-            siblings = [
+            siblings = (
                     i for i in self.parentItem().childItems()
-                    if isinstance(i, HorizontalSymbol) and i is not self]
+                    if isinstance(i, HorizontalSymbol) and i is not self)
             for sibling in siblings:
                 if hasattr(sibling, 'lastBranchItem'):
                     # Disconnect the connection below
@@ -1143,7 +1163,7 @@ class HorizontalSymbol(Symbol):
                     sibling.moveBy(width if self.x() <= sibling.x()
                             else -width, 0)
                     undoCmd = MoveSymbol(sibling, oldSibPos, sibling.pos())
-                    self.scene().undoStack.push(undoCmd)
+                    self.scene().undo_stack.push(undoCmd)
                     sibling.cam(sibling.pos(), sibling.pos())
         super(HorizontalSymbol, self).cam(oldPos, newPos, ignore)
         # Recursively call the cam of the parent
