@@ -72,13 +72,18 @@ dv = None
 tmpVar = 0
 
 # Special SDL operators
-special_operators = {'length': {'type': {'Kind': 'IntegerType'},
-    'paramsInOrdered': ['seqof_ref'], 'paramsOutOrdered': [],
-    'in': {'seqof_ref': {'type': '_SEQOF'}}},
+special_operators = {
+    'length': {'type': {'Kind': 'IntegerType'},
+        'paramsInOrdered': ['seqof_ref'], 'paramsOutOrdered': [],
+        'in': {'seqof_ref': {'type': '_SEQOF'}}},
+    'write': {'type': {'Kind': 'IntegerType'},
+        'paramsInOrdered': None, 'paramsOutOrdered': None},
+    'writeln': {'type': {'Kind': 'IntegerType'},
+        'paramsInOrdered': None, 'paramsOutOrdered': None},
     'present': {'type': {'Kind': 'EnumeratedType', 'extraAttr': 'presentKind',
         'EnumValues': {}},
-    'paramsInOrdered': ['choice_ref'], 'paramsOutOrdered': [],
-    'in': {'choice_ref': {'type': '_CHOICE'}}}}
+        'paramsInOrdered': ['choice_ref'], 'paramsOutOrdered': [],
+        'in': {'choice_ref': {'type': '_CHOICE'}}}}
 
 
 def getInputString(root):
@@ -119,41 +124,49 @@ def find_basic_type(aType):
 
 def check_and_fix_op_params(op_name, expr_list, process):
     '''
-        Verify and/or set the type of a procedure/operator parameters
+        Verify and/or set the type of a procedure/operator/output parameters
         Return false if there is a type mismatch
     '''
     # (1) Find the signature of the function
-    o = None
-    for key, item in process.operators.viewitems():
-        if o.lower() == op_name.lower():
+    for key, item in process.outputSignals.viewitems():
+        if key.lower() == op_name.lower():
             break
     else:
-        for key, item in process.procedures.viewitems():
+        for key, item in process.operators.viewitems():
             if key.lower() == op_name.lower():
                 break
         else:
-            for key, item in special_operators.viewitems():
+            for key, item in process.procedures.viewitems():
                 if key.lower() == op_name.lower():
                     break
             else:
-                print('[ERROR] op not found')
-                return False
+                for key, item in special_operators.viewitems():
+                    if key.lower() == op_name.lower():
+                        break
+                else:
+                    print('[ERROR] Operator/output/procedure not found: '
+                            + op_name)
+                    return False
     # (2) Build a list of all input and output params from the function spec
-    in_params = item.get('paramsInOrdered') or []
-    out_params= item.get('paramsOutOrdered') or []
+    in_params = item.get('paramsInOrdered')
+    out_params = item.get('paramsOutOrdered')
     #all_params = in_params.copy().update(out_params)
     # (3) Check that the number of parameters matches
-    if len(expr_list) != len(in_params + out_params):
-        print('[ERROR] Wrong number of params in op call')
+    if(in_params is not None and out_params is not None and
+            len(expr_list) != len(in_params + out_params)):
+        print('[ERROR] Wrong number of params in op call ' + op_name)
         return False
     # (4) Check each individual parameter type
     for idx, p in enumerate(expr_list):
-        select = ('paramsOutOrdered', 'out', idx-len(in_params)) if (
-                idx >= len(in_params)) else ('paramsInOrdered', 'in', idx)
+        try:
+            select = ('paramsOutOrdered', 'out', idx - len(in_params)) if (
+                    idx >= len(in_params)) else ('paramsInOrdered', 'in', idx)
 
-        # Get parameter name and type name from the Interface view:
-        paramName = item[select[0]][select[2]]
-        paramType = item[select[1]][paramName].get('type')
+            # Get parameter name and type name from the Interface view:
+            paramName = item[select[0]][select[2]]
+            paramType = item[select[1]][paramName].get('type')
+        except:
+            paramType = None
 
         # Try to set the type if it is missing
         # This is needed because some internal SDL operators (Length, Present)
@@ -170,7 +183,8 @@ def check_and_fix_op_params(op_name, expr_list, process):
                 p.exprType = {'Kind': 'ReferenceType',
                         'ReferencedTypeName': paramType}
             else:
-                print('[ERROR] Unable to determine op type')
+                print('[ERROR] Unable to determine param type: ' +
+                        p.inputString + '(in operator ' + op_name + ')')
                 return False
     return True
 
@@ -252,7 +266,8 @@ def isOfCompatibleType(primary, typeRef, process):
                 else:
                     # If the user field is a raw value
                     if (primary.sequence[field].kind == 'primary'
-                            and primary.sequence[field].var.kind != 'expression'
+                            and primary.sequence[field].var.kind
+                                                        != 'expression'
                             and primary.sequence[field].exprType['Kind'] in
                             ('UnknownType', 'IntegerType', 'RealType',
                                 'BooleanType', 'EnumeratedType')):
@@ -312,7 +327,7 @@ def isOfCompatibleType(primary, typeRef, process):
         print '...It is!'
         return True
     elif primary.kind == 'stringLiteral':
-        print 'OCTET STRING NOT SUPPORTED YET'
+        print 'Strings not fully supported yet'
         return True  # Unsupported OCTET STRING
     elif primary.kind == 'ifThenElse':
         # check that IF expr returns BOOL, and that Then and Else expressions
@@ -376,13 +391,14 @@ def compare_types(typeA, typeB):
             except:
                 # Type not in dataview is checked at DCL level
                 # no need to raise another error
-                pass
+                # but return to avoid infinite loop
+                return False
     while B['Kind'] == 'ReferenceType':
             refsetB.append(B['ReferencedTypeName'])
             try:
                 B = dataView[B['ReferencedTypeName']]['type']
             except:
-                pass
+                return False
     if set(refsetA).intersection(refsetB):
         # Types share a common name: they are identical
         # This cover structured types and enumerations
@@ -466,16 +482,17 @@ def findType(path, process):
                             result = special_operators[main.lower()]['type']
                             # Special operators require type elaboration
                             if main.lower() == 'present':
-                                # present(choiceType): must return an enumerated
+                                # present(choiceType): must return an enum
                                 # with elements being the choice options
                                 param, = path[1].get('procParams') or [None]
                                 if not param:
                                     errors.append('Missing parameter'
                                             ' in PRESENT clause')
                                 else:
-                                    param_type = find_basic_type(param.exprType)
+                                    param_type = find_basic_type(
+                                                                param.exprType)
                                     if param_type.get('Kind') != 'ChoiceType':
-                                        errors.append('PRESENT clause parameter'
+                                        errors.append('PRESENT parameter'
                                                 ' must be a CHOICE type')
                                     else:
                                         result['EnumValues'] = (param_type.
@@ -630,6 +647,8 @@ class sdlScc_AST:
                 prim.kind = 'octetString'
                 warnings.append('Octet string literal not supported yet')
             elif child.type == PARAMS:
+                # Cover parameters of operator calls within a task
+                # but not parameters of output or procedure calls
                 if not firstId:
                     errors.append(
                             'Ground expression cannot have index or params: ' +
@@ -671,6 +690,9 @@ class sdlScc_AST:
             prim.primType, err, warn = findType(prim.primaryId, self.process)
             errors.extend(err)
             warnings.extend(warn)
+        elif prim.kind == 'stringLiteral':
+            # Preliminary support of string types (used at least in WRITE(LN)
+            prim.primType = {'Kind': 'StringType'}
         return prim, errors, warnings
 
     def primary(self, root):
@@ -811,12 +833,12 @@ class sdlScc_AST:
                             expr.left.var, expr.right.exprType, self.process)))
                 if not check:
                     errors.append('Incompatible types in expression: left (' +
-                            expr.left.inputString + ', type= '+
+                            expr.left.inputString + ', type= ' +
                             leftExprTypeName + '), right (' +
                             expr.right.inputString +
                             ', type= ' +
                             rightExprTypeName + ') (line ' +
-                            str(expr.line)+ ')')
+                            str(expr.line) + ')')
                 else:
                     if rightIsRaw:
                         expr.right.exprType = expr.left.exprType
@@ -825,11 +847,11 @@ class sdlScc_AST:
             elif not compare_types(expr.left.exprType, expr.right.exprType):
                 errors.append('!Incompatible types in expression: left (' +
                         expr.left.inputString + ', type= ' +
-                        leftExprTypeName + '), right ('+
+                        leftExprTypeName + '), right (' +
                         expr.right.inputString +
                         ', type= ' +
                         rightExprTypeName +
-                        ') (line ' + str(expr.line)+ ')')
+                        ') (line ' + str(expr.line) + ')')
         if root.type in (IN, AND, EQ, NEQ, GT, GE, LT, LE, OR, XOR, AND, IN):
             expr.exprType = {'Kind': 'BooleanType'}
         elif root.type in (PLUS, ASTERISK, DASH, APPEND, REM, MOD):
@@ -1000,7 +1022,7 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(child)
                 i.coordinates = [x, y, w, h]
                 if parent:
-                    i.absCoordinates = [a+b for a, b in
+                    i.absCoordinates = [a + b for a, b in
                             zip(i.coordinates[0:2],
                                 parent.absCoordinates[0:2])]
             elif child.type == INPUTLIST:
@@ -1051,7 +1073,7 @@ class sdlScc_AST:
                 # The reference is an index to process.transitions table
                 self.process.transitions.append(transition)
                 for inp in i.inputlist:
-                    i.inputlist[inp]=len(self.process.transitions) - 1
+                    i.inputlist[inp] = len(self.process.transitions) - 1
             elif child.type == COMMENT:
                 i.comment, _, __ = self.end(child)
             elif child.type == HYPERLINK:
@@ -1073,7 +1095,7 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(child)
                 s.coordinates = [x, y, w, h]
                 if parent:
-                    s.absCoordinates = [a+b for a, b in
+                    s.absCoordinates = [a + b for a, b in
                             zip(s.coordinates[0:2],
                                 parent.absCoordinates[0:2])]
                 else:
@@ -1118,7 +1140,7 @@ class sdlScc_AST:
 
     def cif(self, root):
         ''' Return the CIF coordinates '''
-        result=[]
+        result = []
         for child in root.getChildren():
             if child.type == DASH:
                 val = -int(child.getChild(0).toString())
@@ -1143,7 +1165,8 @@ class sdlScc_AST:
                 map(errors.append, err)
                 map(warnings.append, warn)
                 self.process.transitions.append(s.transition)
-                self.process.mapping['START'] = len(self.process.transitions)-1
+                self.process.mapping['START'] = len(
+                                          self.process.transitions) - 1
             elif child.type == COMMENT:
                 s.comment, _, __ = self.end(child)
             elif child.type == HYPERLINK:
@@ -1191,10 +1214,18 @@ class sdlScc_AST:
                 body['outputName'] = child.toString()
             elif child.type == PARAMS:
                 body['params'], err, warn = self.expression_list(child)
+                # here we must check/set the type of each param
+                if not check_and_fix_op_params(
+                        body.get('outputName') or '',
+                        body['params'],
+                        self.process):
+                    errors.append('Wrong parameter(s) in operation call '
+                            + getInputString(root))
                 errors.extend(err)
                 warnings.extend(warn)
             else:
-                warnings.append('Unsupported output body type:' + str(child.type))
+                warnings.append('Unsupported output body type:' +
+                        str(child.type))
         if body.get('params'):
             body['tmpVars'] = []
             global tmpVar
@@ -1215,7 +1246,7 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(child)
                 o.coordinates = [x, y, w, h]
                 if parent:
-                    o.absCoordinates = [a+b for a, b in
+                    o.absCoordinates = [a + b for a, b in
                             zip(o.coordinates[0:2],
                                 parent.absCoordinates[0:2])]
             elif child.type == OUTPUT_BODY:
@@ -1249,7 +1280,7 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(child)
                 ans.coordinates = [x, y, w, h]
                 if parent:
-                    ans.absCoordinates = [a+b for a, b in
+                    ans.absCoordinates = [a + b for a, b in
                             zip(ans.coordinates[0:2],
                                 parent.absCoordinates[0:2])]
             elif child.type == CLOSED_RANGE:
@@ -1306,7 +1337,7 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(child)
                 dec.coordinates = [x, y, w, h]
                 if parent:
-                    dec.absCoordinates = [a+b for a, b in
+                    dec.absCoordinates = [a + b for a, b in
                             zip(dec.coordinates[0:2],
                                 parent.absCoordinates[0:2])]
             elif child.type == QUESTION:
@@ -1373,7 +1404,7 @@ class sdlScc_AST:
                     if c.type == CIF:
                         x, y, w, h = self.cif(c)
                         a.coordinates = [x, y, w, h]
-                        a.absCoordinates = [u+v for u, v in
+                        a.absCoordinates = [u + v for u, v in
                                 zip(a.coordinates[0:2],
                                     dec.absCoordinates[0:2])]
                     elif c.type == TRANSITION:
@@ -1401,7 +1432,7 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(term)
                 t.coordinates = [x, y, w, h]
                 if parent:
-                    t.absCoordinates = [a+b for a, b in
+                    t.absCoordinates = [a + b for a, b in
                             zip(t.coordinates[0:2],
                                 parent.absCoordinates[0:2])]
             elif term.type == LABEL:
@@ -1588,7 +1619,7 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(child)
                 t.coordinates = [x, y, w, h]
                 if parent:
-                    t.absCoordinates = [a+b for a, b in
+                    t.absCoordinates = [a + b for a, b in
                             zip(t.coordinates[0:2],
                                 parent.absCoordinates[0:2])]
             elif child.type == TASK_BODY:
@@ -1621,8 +1652,9 @@ class sdlScc_AST:
                 x, y, w, h = self.cif(child)
                 t.coordinates = [x, y, w, h]
                 if parent:
-                    t.absCoordinates = [a+b for a, b in
-                            zip(t.coordinates[0:2], parent.absCoordinates[0:2])]
+                    t.absCoordinates = [a + b for a, b in
+                            zip(t.coordinates[0:2],
+                                parent.absCoordinates[0:2])]
             elif child.type == ID:
                 t.inputString = getInputString(child)
                 t.line = child.getLine()
