@@ -25,17 +25,17 @@ from genericSymbols import(
 
 from PySide.QtCore import Qt, QPoint, QRect
 from PySide.QtGui import(QPainterPath, QBrush, QColor, QPolygon,
-        QRadialGradient, QGraphicsItemGroup)
+        QRadialGradient)
 
 import ogParser
 import ogAST
 
-inputCompletionList = []
-outputCompletionList = []
-stateCompletionList = []
-typesCompletionList = []
-varCompletionList = []
-procCompletionList = []
+#inputCompletionList = []
+#outputCompletionList = []
+#stateCompletionList = []
+#typesCompletionList = []
+#varCompletionList = []
+#procCompletionList = []
 
 
 class Input(HorizontalSymbol):
@@ -90,9 +90,9 @@ class Input(HorizontalSymbol):
                 << QPoint(0, 0)
         return polygon
 
-    def completionList(self):
-        ''' Return the list of defined INPUTs for the autocompletion '''
-        return inputCompletionList
+#    def completionList(self):
+#        ''' Return the list of defined INPUTs for the autocompletion '''
+#        return inputCompletionList
 
     def pr(self):
         ''' Return the PR notation of the single INPUT sybol '''
@@ -149,9 +149,9 @@ class Output(VerticalSymbol):
                 << QPoint(0, 0)
         return polygon
 
-    def completionList(self):
-        ''' Return the list of defined OUTPUTs for the autocompletion '''
-        return outputCompletionList
+#    def completionList(self):
+#        ''' Return the list of defined OUTPUTs for the autocompletion '''
+#        return outputCompletionList
 
     def __repr__(self):
         ''' Return the text corresponding to the SDL PR notation '''
@@ -182,25 +182,48 @@ class Decision(VerticalSymbol):
                 y=coord_y, hyperlink=ast.hyperlink)
         self.setBrush(QColor(255, 255, 202))
         self.minDistanceToSymbolAbove = 0
-        self.terminalSymbol = False
+        # Can be a terminal symbol if all branches are terminated!
+        #self.terminalSymbol = False
         self.commonName = 'decision'
         self.parser = ogParser
         self.text_alignment = Qt.AlignHCenter
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
+    @property
+    def terminalSymbol(self):
+        '''
+            Compute dynamically if the item is terminal by checking
+            if all its branches end with a terminator
+        '''
+        for branch in self.branches():
+            if not branch.lastBranchItem.terminalSymbol:
+                return False
+        else:        
+            return True
+
+#    @terminalSymbol.setter
+#    def terminalSymbol(self, value):
+#        ''' Discard the setter '''
+#        pass
+
+    def branches(self):
+        ''' Return the list of decision answers (as a generator) '''
+        return (branch for branch in self.childSymbols()
+                if isinstance(branch, DecisionAnswer))
+
     def __str__(self):
         ''' Return the text string as entered by the user '''
         return str(self.text)
 
     def checkSyntax(self):
         ''' Redefined function, to check only the symbol, not recursively '''
-        _, syntaxErrors, __, ___, ____ = self.parser.parseSingleElement(
+        _, syntax_errors, __, ___, ____ = self.parser.parseSingleElement(
                 self.commonName, self.pr(recursive=False))
         try:
-            self.scene().reportErrors(syntaxErrors)
+            self.scene().reportErrors(syntax_errors)
         except:
-            print('[SYNTAX ERROR]', '\n'.join(syntaxErrors))
+            print('[SYNTAX ERROR]', '\n'.join(syntax_errors))
 
     def computePolygon(self, width, height):
         ''' Define polygon points to draw the symbol '''
@@ -223,54 +246,61 @@ class Decision(VerticalSymbol):
             # Connection point update is only relevant for rendering
             # (i.e. if symbol is in a scene) - ignore otherwise
             return
-        answers = filter(lambda c:
-                isinstance(c, DecisionAnswer), self.childSymbols())
-        newY = 0
-        newX = self.boundingRect().width() / 2
-        if answers:
-            for branch in answers:
-                lastCnx = []
-                lastCmt = None
-                last = branch.lastBranchItem
-                # Coordinates of the last item of a branch
-                # in Decision item coordinates:
-                lastPosY = self.mapFromScene(0, last.scenePos().y()).y()
-                lastCnx = [c for c in last.childItems() if
-                        isinstance(c, Connection) and not
-                        isinstance(c.child, Comment)]
+        new_y = 0
+        new_x = self.boundingRect().width() / 2
+        answers = False
+        for branch in self.branches():
+            answers = True
+            last_cnx = None
+            last = branch.lastBranchItem
+            # Coordinates of the last item of a branch
+            # in Decision item coordinates:
+            last_pos_y = self.mapFromScene(0, last.scenePos().y()).y()
+            try:
                 # To compute the branch length, we must keep only the symbols,
                 # so we must remove the last connection
-                if lastCnx:
-                    lastCnx[0].setParentItem(None)
-                if last.comment:
-                    # Disconnect the comment of the last item so that
-                    # it's size is not taken into account
-                    lastCmt = last.comment
-                    last.comment.setParentItem(None)
-                    last.comment.connection.setParentItem(None)
-                group = QGraphicsItemGroup(last.parentItem())
-                group.addToGroup(last)
-                groupHeight = group.boundingRect().height()
-                self.scene().destroyItemGroup(group)
-                branchLen = lastPosY + groupHeight
-                if lastCnx:
-                    lastCnx[0].setParentItem(last)
-                if lastCmt:
-                    # Reconnect the comment of the last item
-                    lastCmt.setParentItem(last)
-                    last.comment.connection.setParentItem(last)
-                # If last item was a decision, use its connection point
-                # position to get the length of the branch:
-                if isinstance(last, Decision):
-                    branchLen = lastPosY + last.connectionPoint.y()
-                # Rounded with int() -> mandatory when view scale has changed
-                newY = int(max(newY, branchLen))
-        else:
-            newY = int(self.boundingRect().height())
-        newY += 15
-        delta = newY - self.connectionPoint.y()
-        self.connectionPoint.setY(newY)
-        self.connectionPoint.setX(newX)
+                last_cnx, = (c for c in last.childItems() if
+                    isinstance(c, Connection) and not
+                    isinstance(c.child, Comment))
+                last_cnx.setParentItem(None)
+            except:
+                pass
+            if last.comment:
+                # Disconnect the comment of the last item so that
+                # it's size is not taken into account
+                last.comment.setParentItem(None)
+                last.comment.connection.setParentItem(None)
+            # Update MP 30/01/13: Don't use QGraphicsItemGroup,
+            # it has bugs (with PySide 1.1.2)
+            # It puts unwanted focus on editable text of other items
+            # group = QGraphicsItemGroup(last.parentItem())
+            # group.addToGroup(last)
+            # groupHeight = group.boundingRect().height()
+            # self.scene().destroyItemGroup(group)
+            # branch_len = last_pos_y + groupHeight
+            branch_len = branch.y() + (
+                    branch.boundingRect() |
+                    branch.childrenBoundingRect()).height()
+            try:
+                last_cnx.setParentItem(last)
+            except:
+                pass
+            if last.comment:
+                # Reconnect the comment of the last item
+                last.comment.setParentItem(last)
+                last.comment.connection.setParentItem(last)
+            # If last item was a decision, use its connection point
+            # position to get the length of the branch:
+            if isinstance(last, Decision):
+                branch_len = last_pos_y + last.connectionPoint.y()
+            # Rounded with int() -> mandatory when view scale has changed
+            new_y = int(max(new_y, branch_len))
+        if not answers:
+            new_y = int(self.boundingRect().height())
+        new_y += 15
+        delta = new_y - self.connectionPoint.y()
+        self.connectionPoint.setY(new_y)
+        self.connectionPoint.setX(new_x)
         child = filter(lambda c: not isinstance
                 (c, (DecisionAnswer, Comment)), self.childSymbols())
         if child:
@@ -287,13 +317,11 @@ class Decision(VerticalSymbol):
                     w=int(self.boundingRect().width()),
                     h=int(self.boundingRect().height()), comment=comment)]
         if recursive:
-            answers = filter(lambda i: isinstance(i, DecisionAnswer),
-                self.childSymbols())
-            for a in answers:
-                if str(a).lower().strip() == 'else':
-                    result.append(repr(a))
+            for answer in self.branches():
+                if str(answer).lower().strip() == 'else':
+                    result.append(repr(answer))
                 else:
-                    result.insert(1, repr(a))
+                    result.insert(1, repr(answer))
         result.append('ENDDECISION;')
         return '\n'.join(result)
 
@@ -569,9 +597,9 @@ class Task(VerticalSymbol):
                 << QPoint(0, 0)
         return polygon
 
-    def completionList(self):
-        ''' Return the list of defined INPUTs for the autocompletion '''
-        return varCompletionList
+#    def completionList(self):
+#        ''' Return the list of defined INPUTs for the autocompletion '''
+#        return varCompletionList
 
     def __repr__(self):
         ''' Return the text corresponding to the SDL PR notation '''
@@ -623,9 +651,9 @@ class ProcedureCall(VerticalSymbol):
                 << QPoint(7, 0)
         return polygon
 
-    def completionList(self):
-        ''' Return the list of defined procedures for the autocompletion '''
-        return procCompletionList
+#    def completionList(self):
+#        ''' Return the list of defined procedures for the autocompletion '''
+#        return procCompletionList
 
     def __repr__(self):
         ''' Return the text corresponding to the SDL PR notation '''
@@ -680,9 +708,9 @@ class TextSymbol(HorizontalSymbol):
                 << QPoint(width, 10)
         return polygon
 
-    def completionList(self):
-        ''' Return the list of defined ASN.1 datatypes for autocompletion '''
-        return typesCompletionList
+#    def completionList(self):
+#        ''' Return the list of defined ASN.1 datatypes for autocompletion '''
+#        return typesCompletionList
 
     def resizeItem(self, rect):
         ''' Text Symbol only resizes in one direction '''
@@ -752,9 +780,9 @@ class State(VerticalSymbol):
         statePath.addRoundedRect(0, 0, width, height, 15, 50)
         return statePath.toFillPolygon()
 
-    def completionList(self):
-        ''' Return the list of defined STATEs for the autocompletion '''
-        return stateCompletionList
+ #   def completionList(self):
+ #       ''' Return the list of defined STATEs for the autocompletion '''
+ #       return stateCompletionList
 
     def get_ast(self):
         ''' Redefinition of the get_ast function for the state '''
